@@ -127,6 +127,91 @@ eh_configtest (const char *conf_file)
   }
 }
 
+static void
+on_sigchld(int x)
+{
+}
+
+static int
+monitor_process()
+{
+  {
+    struct sigaction sact;
+    sigset_t sset;
+    sact.sa_handler = on_sigchld;
+    sact.sa_flags = 0;
+    sigemptyset(&sset);
+    sigaddset(&sset, SIGINT);
+    sigaddset(&sset, SIGTERM);
+    sigaddset(&sset, SIGHUP);
+    sigaddset(&sset, SIGCHLD);
+    if (sigprocmask(SIG_BLOCK, &sset, NULL) < 0) {
+      eh_log (EH_LOG_FATAL, "monitor: failed to set signal mask");
+      _exit(-1);
+    }
+    if (sigaction(SIGCHLD, &sact, NULL) < 0) {
+      eh_log (EH_LOG_FATAL, "monitor: failed to set SIGCHLD handler");
+      _exit(-1);
+    }
+  }
+  while (1) {
+    int waiting_child = 1;
+    pid_t pid = fork();
+    if (pid < 0) {
+      eh_log(EH_LOG_FATAL, "monitor: failed to fork a worker process");
+      _exit(-1);
+    }
+    if (pid == 0) {
+      /* worker */
+      return 0;
+    }
+    /* monitor process */
+    while (waiting_child) {
+      sigset_t sset;
+      pid_t p = 0;
+      int chldst = 0;
+      int sig = 0;
+      sigemptyset(&sset);
+      sigaddset(&sset, SIGINT);
+      sigaddset(&sset, SIGTERM);
+      sigaddset(&sset, SIGHUP);
+      sigaddset(&sset, SIGCHLD);
+      sigwait(&sset, &sig);
+      switch (sig) {
+      case SIGINT:
+      case SIGTERM:
+      case SIGHUP:
+	eh_log(EH_LOG_INFO, "monitor: sending signal %d to the worker", sig);
+	if (kill(pid, sig) < 0) {
+	  eh_log(EH_LOG_WARNING,
+		 "monitor: failed to send signal(pid=%d,err=%d)", pid, errno);
+	}
+	break;
+      case SIGCHLD:
+	p = waitpid(pid, &chldst, WNOHANG);
+	if (p < 0) {
+	  eh_log(EH_LOG_WARNING, "monitor: failed to get child status(err=%d)",
+	  	 errno);
+	} else if (p == 0) {
+	  ;
+	} else {
+	  if (WIFEXITED(chldst)) {
+	    int st = WEXITSTATUS(chldst);
+	    eh_log(EH_LOG_INFO, "monitor: worker exits with status %d", st);
+	    _exit(st);
+	  } else if (WIFSIGNALED(chldst)) {
+	    eh_log(EH_LOG_INFO, "monitor: worker has signaled(sig=%d)",
+		   WTERMSIG(chldst));
+	    waiting_child = 0;
+	  }
+	}
+	break;
+      } /* switch (sig) */
+    } /* while (waiting_child) */
+  }
+  _exit(-1);
+}
+
 int main (int argc, char **argv)
 {
   eh_app_t app;
@@ -250,6 +335,13 @@ int main (int argc, char **argv)
   /* drop root privilege */
   eh_app_chroot_setuid (&app);
 
+  /* fork monitor process */
+  if (!nodaemon && app.econf->startservers == 1) {
+    monitor_process();
+    eh_fd_getpid ();
+    eh_log_getpid ();
+  }
+
   /* fork worker processes */
   eh_app_start_workers (&app);
 
