@@ -480,6 +480,15 @@ eh_fd_set_data (eh_fd_t *ef, void (*on_event)(eh_fd_t *ef),
   ef->user_data = user_data;
 }
 
+void
+eh_fd_reset_pid (eh_fd_t *ef)
+{
+  if (method == eh_fd_multiplexing_method_rtsignal) {
+    fcntl (ef->pfd.fd, F_SETOWN, my_pid);
+    eh_log (EH_LOG_INFO, "%d: pid=%d", ef->pfd.fd, my_pid);
+  }
+}
+
 static int
 eh_fd_has_masked_event (eh_fd_t *ef)
 {
